Attribute VB_Name = "modEncodeQuery"
'' Character Encode Query for Passolo
'' (c) 2014 - 2015 by wanfu (Last modified on 2015.11.08)

'' Command Line Format: Command String,Codepage,Escape
'' Command: Name of this Macros file
'' String: String to be converted. If include ",", use "string" format
'' Codepage: Numeric value of codepage
'' Escape: 0 = Not escape, 1 = Escape

Option Explicit

Private Enum KnownCodePage
	CP_UNKNOWN = -1
	CP_ACP = 0
	CP_OEMCP = 1
	CP_MACCP = 2
	CP_THREAD_ACP = 3
	CP_SYMBOL = 42
' ARABIC
	CP_AWIN = 101 ' Bidi Windows codepage
	CP_709 = 102 ' MS-DOS Arabic Support 	CP 709
	CP_720 = 103 ' MS-DOS Arabic Support 	CP 720
	CP_A708 = 104 ' ASMO 708
	CP_A449 = 105 ' ASMO 449+
	CP_TARB = 106 ' MS Transparent Arabic
	CP_NAE = 107 ' Nafitha Enhanced Arabic Char Set
	CP_V4 = 108 ' Nafitha v 4.0
	CP_MA2 = 109 ' Mussaed Al Arabi (MA/2) 	CP 786
	CP_I864 = 110 ' IBM Arabic Supplement 	CP 864
	CP_A437 = 111 ' Ansi 437 codepage
	CP_AMAC = 112 ' Macintosh Code Page
' HEBREW
	CP_HWIN = 201 ' Bidi Windows codepage
	CP_862I = 202 ' IBM Hebrew Supplement 	CP 862
	CP_7BIT = 203 ' IBM Hebrew Supplement 	CP 862 Folded
	CP_ISO = 204 ' ISO Hebrew 8859-8 Character Set
	CP_H437 = 205 ' Ansi 437 codepage
	CP_HMAC = 206 ' Macintosh Code Page
' CODE PAGES
	CP_OEM_437 = 437
	CP_ARABICDOS = 708
	CP_DOS720 = 720
	CP_DOS737 = 737
	CP_DOS775 = 775
	CP_IBM850 = 850
	CP_IBM852 = 852
	CP_DOS861 = 861
	CP_DOS862 = 862
	CP_IBM866 = 866
	CP_DOS869 = 869
	CP_THAI = 874
	CP_EBCDIC = 875
	CP_JAPAN = 932
	CP_CHINA = 936
	CP_KOREA = 949
	CP_TAIWAN = 950
' UNICODE
	CP_UNICODELITTLE = 1200
	CP_UNICODEBIG = 1201
' CODE PAGES
	CP_EASTEUROPE = 1250
	CP_RUSSIAN = 1251
	CP_WESTEUROPE = 1252
	CP_GREEK = 1253
	CP_TURKISH = 1254
	CP_HEBREW = 1255
	CP_ARABIC = 1256
	CP_BALTIC = 1257
	CP_VIETNAMESE = 1258
' KOREAN
	CP_JOHAB = 1361
' MAC
	CP_MAC_ROMAN = 10000
	CP_MAC_JAPAN = 10001
	CP_MAC_ARABIC = 10004
	CP_MAC_GREEK = 10006
	CP_MAC_CYRILLIC = 10007
	CP_MAC_LATIN2 = 10029
	CP_MAC_TURKISH = 10081
' CODE PAGES
	CP_CHINESECNS = 20000
	CP_CHINESEETEN = 20002
	CP_IA5WEST = 20105
	CP_IA5GERMAN = 20106
	CP_IA5SWEDISH = 20107
	CP_IA5NORWEGIAN = 20108
	CP_ASCII = 20127
	CP_RUSSIANKOI8R = 20866
	CP_RUSSIANKOI8U = 21866
	CP_ISOLATIN1 = 28591
	CP_ISOEASTEUROPE = 28592
	CP_ISOTURKISH = 28593
	CP_ISOBALTIC = 28594
	CP_ISORUSSIAN = 28595
	CP_ISOARABIC = 28596
	CP_ISOGREEK = 28597
	CP_ISOHEBREW = 28598
	CP_ISOTURKISH2 = 28599
	CP_ISOLATIN9 = 28605
	CP_HEBREWLOG = 38598
	CP_USER = 50000
	CP_AUTOALL = 50001
	CP_JAPANNHK = 50220
	CP_JAPANESC = 50221
	CP_JAPANISO = 50222
	CP_KOREAISO = 50225
	CP_TAIWANISO = 50227
	CP_CHINAISO = 50229
	CP_AUTOJAPAN = 50932
	CP_AUTOCHINA = 50936
	CP_AUTOKOREA = 50949
	CP_AUTOTAIWAN = 50950
	CP_AUTORUSSIAN = 51251
	CP_AUTOGREEK = 51253
	CP_AUTOARABIC = 51256
	CP_JAPANEUC = 51932
	CP_CHINAEUC = 51936
	CP_KOREAEUC = 51949
	CP_TAIWANEUC = 51950
	CP_CHINAHZ = 52936
	CP_GB18030 = 54936
' UNICODE
	CP_UTF7 = 65000
	CP_UTF8 = 65001
End Enum

'ҳת
Private Declare Function MultiByteToWideChar Lib "kernel32.dll" ( _
	ByVal CodePage As Long, _
	ByVal dwFlags As Long, _
	ByRef lpMultiByteStr As Any, _
	ByVal cchMultiByte As Long, _
	ByVal lpWideCharStr As Long, _
	ByVal cchWideChar As Long) As Long
Private Declare Function WideCharToMultiByte Lib "kernel32.dll" ( _
	ByVal CodePage As Long, _
	ByVal dwFlags As Long, _
	ByVal lpWideCharStr As Long, _
	ByVal cchWideChar As Long, _
	ByRef lpMultiByteStr As Any, _
	ByVal cchMultiByte As Long, _
	ByVal lpDefaultChar As Long, _
	ByVal lpUsedDefaultChar As Long) As Long
Private Declare Function GetACP Lib "kernel32.dll" () As Long

Private MsgList() As String
Private Const Version = "2015.11.08"
Private Const Build = "151108"
Private Const TextJoinStr = vbCrLf


' 
Sub Main
	On Error GoTo SysErrorMsg
	If GetMsgList(MsgList) = False Then GoTo SysErrorMsg
	Begin Dialog UserDialog 660,322,Replace$(Replace$(MsgList(12),"%v",Version),"%b",Build),.MainDlgFunc ' %GRID:10,7,1,1
		CheckBox 60,49,30,21,"",.CheckBox
		Text 10,7,80,14,MsgList(13),.Text,1
		Text 10,112,80,14,MsgList(14),.ANSI,1
		Text 10,154,80,14,MsgList(15),.Unicode,1
		Text 10,196,80,14,MsgList(16),.UniBigEnd,1
		Text 10,238,80,14,MsgList(17),.UTF8,1
		Text 10,280,80,14,MsgList(18),.UTF7,1
		Text 10,21,80,14,"",.TextLength,1
		Text 10,126,80,14,"",.ANSILength,1
		Text 10,168,80,14,"",.UnicodeLength,1
		Text 10,210,80,14,"",.UniBigEndLength,1
		Text 10,252,80,14,"",.UTF8Length,1
		Text 10,294,80,14,"",.UTF7Length,1
		TextBox 100,7,490,70,.TextBox,1
		TextBox 100,112,490,35,.ANSIBox,1
		TextBox 100,154,490,35,.UnicodeBox,1
		TextBox 100,196,490,35,.UniBigEndBox,1
		TextBox 100,238,490,35,.UTF8Box,1
		TextBox 100,280,490,35,.UTF7Box,1
		DropListBox 100,84,280,21,MsgList(),.CPNameList
		DropListBox 100,84,280,21,MsgList(),.CPValueList
		CheckBox 460,84,130,21,MsgList(19),.ConvetBox
		PushButton 600,7,50,28,MsgList(20),.AboutButton
		PushButton 600,49,50,28,MsgList(21),.TextButton
		PushButton 600,112,50,28,MsgList(21),.ANSIButton
		PushButton 600,154,50,28,MsgList(21),.UnicodeButton
		PushButton 600,196,50,28,MsgList(21),.UniBigEndButton
		PushButton 600,238,50,28,MsgList(21),.UTF8Button
		PushButton 600,280,50,28,MsgList(21),.UTF7Button
		CancelButton 10,49,30,21,.CancelButton
	End Dialog
	Dim dlg As UserDialog
	If Dialog(dlg) = 0 Then Exit Sub
	Exit Sub
	SysErrorMsg:
	If Err.Source <> "ExitSub" Then Call sysErrorMassage(Err,0)
End Sub


'Ի
Private Function MainDlgFunc(DlgItem$, Action%, SuppValue&) As Boolean
	Dim i As Integer,Temp As String
	DlgValue "CheckBox",0
	Select Case Action%
	Case 1 ' Ի򴰿ڳʼ
		DlgVisible "CheckBox",False
		DlgVisible "CancelButton",False
		DlgVisible "CPValueList",False
		ReDim TempList(17) As String
		TempList(0) = CP_OEMCP & " - " & MsgList(23)		'ϵͳĬ
		TempList(1) = CP_MACCP & " - " & MsgList(24)
		TempList(2) = CP_THREAD_ACP & " - " & MsgList(25)
		TempList(3) = CP_WESTEUROPE & " - " & MsgList(26)	' 1 (ANSI) = 1252
		TempList(4) = CP_EASTEUROPE & " - " & MsgList(27)	' 2 (ŷ) = 1250
		TempList(5) = CP_RUSSIAN & " - " & MsgList(28)		' (˹) = 1251
		TempList(6) = CP_GREEK & " - " & MsgList(29)		'ϣ = 1253
		TempList(7) = CP_TURKISH & " - " & MsgList(30)		' 5 () = 1254
		TempList(8) = CP_HEBREW & " - " & MsgList(31)		'ϣ = 1255
		TempList(9) = CP_ARABIC & " - " & MsgList(32)		' = 1256
		TempList(10) = CP_BALTIC & " - " & MsgList(33)		'޵ĺ = 1257
		TempList(11) = CP_VIETNAMESE & " - " & MsgList(34)	'Խ = 1258
		TempList(12) = CP_JAPAN & " - " & MsgList(35)		' = 932
		TempList(13) = CP_CHINA & " - " & MsgList(36)		' = 936
		TempList(14) = CP_GB18030 & " - " & MsgList(37)		' = 54936
		TempList(15) = CP_KOREA & " - " & MsgList(38)		' = 949
		TempList(16) = CP_TAIWAN & " - " & MsgList(39) 		' = 950
		TempList(17) = CP_THAI & " - " & MsgList(40)		'̩ = 874
		DlgListBoxArray "CPNameList",TempList()
		For i = 0 To 17
			TempList(i) = Trim$(Split(TempList(i),"-")(0))
		Next i
		DlgListBoxArray "CPValueList",TempList()
		If Command$ = "" Then
			DlgText "CPValueList",CStr(GetACP)
		Else
			TempList = SplitArgument(Command$,",",3)
			Select Case UBound(TempList)
			Case 0
				DlgText "TextBox",TempList(0)
				DlgText "CPValueList",CStr(GetACP)
			Case 1
				DlgText "TextBox",TempList(0)
				DlgText "CPValueList",TempList(1)
			Case 2
				DlgText "TextBox",TempList(0)
				DlgText "CPValueList",TempList(1)
				DlgValue "ConvetBox",StrToLong(TempList(2))
			End Select
		End If
		If DlgText("CPValueList") = "" Then DlgText "CPValueList",CStr(GetACP)
		DlgValue "CPNameList",DlgValue("CPValueList")
		If DlgText("TextBox") <> "" Then
			Temp = DlgText("TextBox")
			If DlgValue("ConvetBox") = 1 Then
				Temp = Convert(Temp)
				'DlgText "TextBox",ReConvert(Temp)
			End If
			ReDim LngList(5),TempList(5) As String
			LngList(0) = Len(Temp)
			LngList(1) = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
			LngList(2) = StrHexLength(Temp,CP_UNICODELITTLE,1)
			LngList(3) = StrHexLength(Temp,CP_UNICODEBIG,1)
			LngList(4) = StrHexLength(Temp,CP_UTF8,1)
			LngList(5) = StrHexLength(Temp,CP_UTF7,1)
			For i = 0 To 5
				TempList(i) = Replace$(Replace$(MsgList(22),"%s",CStr$(LngList(i))),"%d",FormatHexStr(Hex$(LngList(i)),2))
			Next i
			DlgText "ANSIBox",Byte2CodeStr(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
			DlgText "UnicodeBox",Byte2CodeStr(StringToByte(Temp,CP_UNICODELITTLE),0,-1)
			DlgText "UniBigEndBox",Byte2CodeStr(StringToByte(Temp,CP_UNICODEBIG),0,-1)
			DlgText "UTF8Box",Byte2CodeStr(StringToByte(Temp,CP_UTF8),0,-1)
			DlgText "UTF7Box",Byte2CodeStr(StringToByte(Temp,CP_UTF7),0,-1)
			DlgText "TextLength",TempList(0)
			DlgText "ANSILength",TempList(1)
			DlgText "UnicodeLength",TempList(2)
			DlgText "UniBigEndLength",TempList(3)
			DlgText "UTF8Length",TempList(4)
			DlgText "UTF7Length",TempList(5)
		End If
	Case 2 ' ֵĻ߰˰ť
		If DlgItem$ = "CancelButton" Then Exit Function
		MainDlgFunc = True ' ֹ°ťرնԻ򴰿
		Select Case DlgItem$
		Case "AboutButton"
			MsgBox Replace$(Replace$(MsgList(41),"%v",Version),"%b",Build),vbOkOnly+vbInformation,MsgList(42)
		Case "TextButton"
			Clipboard DlgText("TextBox")
		Case "ANSIButton"
			Clipboard DlgText("ANSIBox")
		Case "UnicodeButton"
			Clipboard DlgText("UnicodeBox")
		Case "UniBigEndButton"
			Clipboard DlgText("UniBigEndBox")
		Case "UTF8Button"
			Clipboard DlgText("UTF8Box")
		Case "UTF7Button"
			Clipboard DlgText("UTF7Box")
		Case "CPNameList"
			DlgValue "CPValueList",DlgValue("CPNameList")
			If DlgText("TextBox") <> "" Then
				Temp = DlgText("TextBox")
				If DlgValue("ConvetBox") = 1 Then Temp = Convert(Temp)
				i = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
				DlgText "ANSIBox",Byte2CodeStr(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
				DlgText "ANSILength",Replace$(Replace$(MsgList(22),"%s",CStr$(i)),"%d",FormatHexStr(Hex$(i),2))
			Else
				DlgText "ANSIBox",""
				DlgText "ANSILength",""
			End If
			DlgValue "CheckBox",1
		Case "ConvetBox"
			If DlgText("TextBox") <> "" Then
				Temp = DlgText("TextBox")
				If DlgValue("ConvetBox") = 1 Then
					Temp = Convert(Temp)
					'DlgText "TextBox",ReConvert(Temp)
				End If
				ReDim LngList(5),TempList(5) As String
				LngList(0) = Len(Temp)
				LngList(1) = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
				LngList(2) = StrHexLength(Temp,CP_UNICODELITTLE,1)
				LngList(3) = StrHexLength(Temp,CP_UNICODEBIG,1)
				LngList(4) = StrHexLength(Temp,CP_UTF8,1)
				LngList(5) = StrHexLength(Temp,CP_UTF7,1)
				For i = 0 To 5
					TempList(i) = Replace$(Replace$(MsgList(22),"%s",CStr$(LngList(i))),"%d",FormatHexStr(Hex$(LngList(i)),2))
				Next i
				DlgText "ANSIBox",Byte2CodeStr(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
				DlgText "UnicodeBox",Byte2CodeStr(StringToByte(Temp,CP_UNICODELITTLE),0,-1)
				DlgText "UniBigEndBox",Byte2CodeStr(StringToByte(Temp,CP_UNICODEBIG),0,-1)
				DlgText "UTF8Box",Byte2CodeStr(StringToByte(Temp,CP_UTF8),0,-1)
				DlgText "UTF7Box",Byte2CodeStr(StringToByte(Temp,CP_UTF7),0,-1)
				DlgText "TextLength",TempList(0)
				DlgText "ANSILength",TempList(1)
				DlgText "UnicodeLength",TempList(2)
				DlgText "UniBigEndLength",TempList(3)
				DlgText "UTF8Length",TempList(4)
				DlgText "UTF7Length",TempList(5)
			Else
				DlgText "ANSIBox",""
				DlgText "UnicodeBox",""
				DlgText "UniBigEndBox",""
				DlgText "UTF8Box",""
				DlgText "UTF7Box",""
				DlgText "TextLength",""
				DlgText "ANSILength",""
				DlgText "UnicodeLength",""
				DlgText "UniBigEndLength",""
				DlgText "UTF8Length",""
				DlgText "UTF7Length",""
			End If
		End Select
	Case 3 ' ıϿı
		If DlgValue("CheckBox") = 1 Then Exit Function
		Select Case DlgItem$
		Case "TextBox"
			If DlgText("TextBox") <> "" Then
				Temp = DlgText("TextBox")
				If DlgValue("ConvetBox") = 1 Then
					Temp = Convert(Temp)
					'DlgText "TextBox",ReConvert(Temp)
				End If
			End If
		Case "ANSIBox"
			If CheckHexStr(Replace$(DlgText("ANSIBox")," ",""),2) = True Then
				DlgText "ANSIBox",UCase$(Replace$(DlgText("ANSIBox"),vbCrLf,""))
				Temp = ByteToString(HexStr2Bytes(Replace$(DlgText("ANSIBox")," ","")),CLng(Trim$(DlgText("CPValueList"))))
				If DlgValue("ConvetBox") = 0 Then
					DlgText "TextBox",Temp
				Else
					DlgText "TextBox",ReConvert(Temp)
				End If
			End If
		Case "UnicodeBox"
			If CheckHexStr(Replace$(DlgText("UnicodeBox")," ",""),4) = True Then
				DlgText "UnicodeBox",UCase$(Replace$(DlgText("UnicodeBox"),vbCrLf,""))
				Temp = ByteToString(HexStr2Bytes(Replace$(DlgText("UnicodeBox")," ","")),CP_UNICODELITTLE)
				If DlgValue("ConvetBox") = 0 Then
					DlgText "TextBox",Temp
				Else
					DlgText "TextBox",ReConvert(Temp)
				End If
			End If
		Case "UniBigEndBox"
			If CheckHexStr(Replace$(DlgText("UniBigEndBox")," ",""),4) = True Then
				DlgText "UniBigEndBox",UCase$(Replace$(DlgText("UniBigEndBox"),vbCrLf,""))
				Temp = ByteToString(HexStr2Bytes(Replace$(DlgText("UniBigEndBox")," ","")),CP_UNICODEBIG)
				If DlgValue("ConvetBox") = 0 Then
					DlgText "TextBox",Temp
				Else
					DlgText "TextBox",ReConvert(Temp)
				End If
			End If
		Case "UTF8Box"
			If CheckHexStr(Replace$(DlgText("UTF8Box")," ",""),2) = True Then
				DlgText "UTF8Box",UCase$(Replace$(DlgText("UTF8Box"),vbCrLf,""))
				Temp = ByteToString(HexStr2Bytes(Replace$(DlgText("UTF8Box")," ","")),CP_UTF8)
				If DlgValue("ConvetBox") = 0 Then
					DlgText "TextBox",Temp
				Else
					DlgText "TextBox",ReConvert(Temp)
				End If
			End If
		Case "UTF7Box"
			If CheckHexStr(Replace$(DlgText("UTF7Box")," ",""),2) = True Then
				DlgText "UTF7Box",UCase$(Replace$(DlgText("UTF7Box"),vbCrLf,""))
				Temp = ByteToString(HexStr2Bytes(Replace$(DlgText("UTF7Box")," ","")),CP_UTF7)
				If DlgValue("ConvetBox") = 0 Then
					DlgText "TextBox",Temp
				Else
					DlgText "TextBox",ReConvert(Temp)
				End If
			End If
		End Select
		If Temp <> "" Then
			ReDim LngList(5),TempList(5) As String
			LngList(0) = Len(Temp)
			LngList(1) = StrHexLength(Temp,CLng(Trim$(DlgText("CPValueList"))),1)
			LngList(2) = StrHexLength(Temp,CP_UNICODELITTLE,1)
			LngList(3) = StrHexLength(Temp,CP_UNICODEBIG,1)
			LngList(4) = StrHexLength(Temp,CP_UTF8,1)
			LngList(5) = StrHexLength(Temp,CP_UTF7,1)
			For i = 0 To 5
				TempList(i) = Replace$(Replace$(MsgList(22),"%s",CStr$(LngList(i))),"%d",FormatHexStr(Hex$(LngList(i)),2))
			Next i
			If DlgItem$ <> "ANSIBox" Then DlgText "ANSIBox",Byte2CodeStr(StringToByte(Temp,CLng(Trim$(DlgText("CPValueList")))),0,-1)
			If DlgItem$ <> "UnicodeBox" Then DlgText "UnicodeBox",Byte2CodeStr(StringToByte(Temp,CP_UNICODELITTLE),0,-1)
			If DlgItem$ <> "UniBigEndBox" Then DlgText "UniBigEndBox",Byte2CodeStr(StringToByte(Temp,CP_UNICODEBIG),0,-1)
			If DlgItem$ <> "UTF8Box" Then DlgText "UTF8Box",Byte2CodeStr(StringToByte(Temp,CP_UTF8),0,-1)
			If DlgItem$ <> "UTF7Box" Then DlgText "UTF7Box",Byte2CodeStr(StringToByte(Temp,CP_UTF7),0,-1)
			DlgText "TextLength",TempList(0)
			DlgText "ANSILength",TempList(1)
			DlgText "UnicodeLength",TempList(2)
			DlgText "UniBigEndLength",TempList(3)
			DlgText "UTF8Length",TempList(4)
			DlgText "UTF7Length",TempList(5)
		Else
			If DlgItem$ <> "TextBox" Then DlgText "TextBox",""
			If DlgItem$ <> "ANSIBox" Then DlgText "ANSIBox",""
			If DlgItem$ <> "UnicodeBox" Then DlgText "UnicodeBox",""
			If DlgItem$ <> "UniBigEndBox" Then DlgText "UniBigEndBox",""
			If DlgItem$ <> "UTF8Box" Then DlgText "UTF8Box",""
			If DlgItem$ <> "UTF7Box" Then DlgText "UTF7Box",""
			DlgText "TextLength",""
			DlgText "ANSILength",""
			DlgText "UnicodeLength",""
			DlgText "UniBigEndLength",""
			DlgText "UTF8Length",""
			DlgText "UTF7Length",""
		End If
	End Select
End Function


'õĴҳUTF-8:65001GB2312:936GB1803054936UTF-765000
Private Function MultiByteToUTF16(UTF8() As Byte, ByVal CodePage As Long) As String
	Dim bufSize As Long,lRet As Long
	'bufSize = MultiByteToWideChar(CodePage, 0&, VarPtr(UTF8(0)), UBound(UTF8) + 1, 0, 0)
	bufSize = MultiByteToWideChar(CodePage, 0&, UTF8(0), UBound(UTF8) + 1, 0, 0)
	'If CodePage = CP_UTF8 Then bufSize = (UBound(UTF8) + 1) * 2 Else bufSize = UBound(UTF8) + 1
	MultiByteToUTF16 = Space(bufSize)
	'lRet = MultiByteToWideChar(CodePage, 0&, VarPtr(UTF8(0)), UBound(UTF8) + 1, StrPtr(MultiByteToUTF16), bufSize)
	lRet = MultiByteToWideChar(CodePage, 0&, UTF8(0), UBound(UTF8)+1, StrPtr(MultiByteToUTF16), bufSize)
	If lRet > 0 Then
		MultiByteToUTF16 = Left$(MultiByteToUTF16, lRet)
	End If
End Function


Private Function UTF16ToMultiByte(ByVal UTF16 As String, ByVal CodePage As Long) As Byte()
	Dim bufSize As Long,lRet As Long
	ReDim arr(0) As Byte
	bufSize = WideCharToMultiByte(CodePage, 0&, StrPtr(UTF16), Len(UTF16), arr(0), 0, 0, 0)
	'If CodePage = CP_UTF8 Then bufSize = 2 * LenB(UTF16) - 1 Else bufSize = LenB(UTF16)
	If bufSize < 1 Then bufSize = 1
	ReDim arr(bufSize - 1) As Byte
	lRet = WideCharToMultiByte(CodePage, 0&, StrPtr(UTF16), Len(UTF16), arr(0), bufSize, 0, 0)
	If lRet > 0 Then
		ReDim Preserve arr(lRet - 1) As Byte
	End If
	UTF16ToMultiByte = arr
End Function


'ȡ Hex 
'StartNo <= EndNo ȡλλ Hex 룬ȡλλ Hex 
Public Function Byte2CodeStr(Bin() As Byte,ByVal StartNo As Long,ByVal EndNo As Long) As String
	Dim i As Long,n As Long
	If StartNo < 0 Then StartNo = UBound(Bin)
	If EndNo < 0 Then EndNo = UBound(Bin)
	Byte2CodeStr = Space$((Abs(EndNo - StartNo) + 1) * 2)
	n = 1
	For i = StartNo To EndNo Step IIf(StartNo <= EndNo,1,-1)
		Mid$(Byte2CodeStr,n,2) = Right$("0" & Hex$(Bin(i)),2)
		n = n + 2
	Next i
End Function


'תƫôΪֽ
Private Function HexStr2Bytes(ByVal HexStr As String) As Byte()
	Dim i As Long,n As Long,Length As Long
	Length = Len(HexStr)
	If Length > 1 Then
		ReDim tempByte(Length \ 2 - 1) As Byte
		For i = 1 To Length Step 2
			tempByte(n) = Val("&H" & Mid$(HexStr,i,2))
			n = n + 1
		Next i
	Else
		ReDim tempByte(0) As Byte
	End If
	HexStr2Bytes = tempByte
End Function


'ַתֽ
Private Function StringToByte(ByVal textStr As String,ByVal CodePage As Long) As Byte()
	If textStr = "" Then
		ReDim StringToByte(0) As Byte
		Exit Function
	End If
	Select Case CodePage
	Case CP_UNICODELITTLE
		StringToByte = textStr
	Case CP_UNICODEBIG
		StringToByte = textStr
		Call LowByte2HighByte(StringToByte)
	Case Else
		StringToByte = UTF16ToMultiByte(textStr,CodePage)	'ָҳתUnicodeַΪANSI
		'StringToByte = StrConv$(textStr,vbFromUnicode)		'ҳתUnicodeַΪANSI
	End Select
End Function


'ֽתַ
Private Function ByteToString(Bytes() As Byte,ByVal CodePage As Long) As String
	Select Case CodePage
	Case CP_UNICODELITTLE
		ByteToString = Bytes
	Case CP_UNICODEBIG
		Call LowByte2HighByte(Bytes)
		ByteToString = Bytes
	Case Else
		ByteToString = MultiByteToUTF16(Bytes,CodePage)
	End Select
End Function


'ÿֽڵĸֽں͵ֽڻ UNICODE LITTLE  UNICODE BIG ֽ໥ת
Private Sub LowByte2HighByte(Bytes() As Byte)
	Dim i As Long,Temp As Byte
	For i = LBound(Bytes) To UBound(Bytes) -1 Step 2
		Temp = Bytes(i)
		Bytes(i) = Bytes(i + 1)
		Bytes(i + 1) = Temp
	Next i
End Sub


'ִǷҪ
Private Function CheckHexStr(ByVal textStr As String,ByVal Length As Integer) As Boolean
	If textStr = "" Then Exit Function
	If (Len(textStr) Mod Length) <> 0 Then Exit Function
	If CheckStr(textStr,"[0-9A-Fa-f]",0,1) = False Then Exit Function
	CheckHexStr = True
End Function


'ִת
Private Function Convert(ByVal ConverString As String) As String
	Convert = ConverString
	If Convert = "" Then Exit Function
	If InStr(Convert,"\") = 0 Then Exit Function
	If InStr(Convert,"\\") Then Convert = Replace$(Convert,"\\","*a!N!d*")
	If InStr(Convert,"\r\n") Then Convert = Replace$(Convert,"\r\n",vbCrLf)
	If InStr(Convert,"\r\n") Then Convert = Replace$(Convert,"\r\n",vbNewLine)
	If InStr(Convert,"\r") Then Convert = Replace$(Convert,"\r",vbCr)
	If InStr(Convert,"\r") Then Convert = Replace$(Convert,"\r",vbNewLine)
	If InStr(Convert,"\n") Then Convert = Replace$(Convert,"\n",vbLf)
	If InStr(Convert,"\b") Then Convert = Replace$(Convert,"\b",vbBack)
	If InStr(Convert,"\f") Then Convert = Replace$(Convert,"\f",vbFormFeed)
	If InStr(Convert,"\v") Then Convert = Replace$(Convert,"\v",vbVerticalTab)
	If InStr(Convert,"\t") Then Convert = Replace$(Convert,"\t",vbTab)
	If InStr(Convert,"\'") Then Convert = Replace$(Convert,"\'","'")
	If InStr(Convert,"\""") Then Convert = Replace$(Convert,"\""","""")
	If InStr(Convert,"\?") Then Convert = Replace$(Convert,"\?","?")
	If InStr(Convert,"\") Then Convert = ConvertB(Convert)
	If InStr(Convert,"\0") Then Convert = Replace$(Convert,"\0",vbNullChar)
	If InStr(Convert,"*a!N!d*") Then Convert = Replace$(Convert,"*a!N!d*","\")
End Function


'ת˽ƻʮת
Private Function ConvertB(ByVal ConverString As String) As String
	Dim i As Long,j As Long,EscStr As String,ConvCode As String
	ConvertB = ConverString
	i = InStr(ConvertB,"\")
	Do While i > 0
		EscStr = Mid$(ConvertB,i,2)
		Select Case EscStr
		Case "\x", "\X"
			ConvCode = Mid$(ConvertB,i + 2,2)
			If CheckStr(UCase$(ConvCode),"[0-9A-Fa-f]",0,1) = True Then
				j = Val("&H" & ConvCode)
				ConvertB = Replace$(ConvertB,EscStr & ConvCode,ChrW(j))
			End If
		Case "\u", "\U"
			ConvCode = Mid$(ConvertB,i + 2,4)
			If CheckStr(UCase$(ConvCode),"[0-9A-Fa-f]",0,1) = True Then
				j = Val("&H" & ConvCode)
				ConvertB = Replace$(ConvertB,EscStr & ConvCode,ChrW(j))
			End If
		Case Is <> ""
			EscStr = "\"
			For j = 3 To 1 Step -1
				ConvCode = Mid$(ConvertB,i + 1,j)
				If CheckStr(ConvCode,"[0-7]",0,1) = True Then
					j = Val("&O" & ConvCode)
					If j > 256 Then
						ConvCode = Left$(ConvCode,2)
						j = Val("&O" & ConvCode)
					End If
					ConvertB = Replace$(ConvertB,EscStr & ConvCode,ChrW(j))
					Exit For
				End If
			Next j
		End Select
		i = InStr(i + 1,ConvertB,"\")
	Loop
End Function


'ִת
Private Function ReConvert(ByVal ConverString As String) As String
	ReConvert = ConverString
	If ReConvert = "" Then Exit Function
	If InStr(ReConvert,"\") Then ReConvert = Replace$(ReConvert,"\","\\")
	If InStr(ReConvert,vbCrLf) Then ReConvert = Replace$(ReConvert,vbCrLf,"\r\n")
	If InStr(ReConvert,vbNewLine) Then ReConvert = Replace$(ReConvert,vbNewLine,"\r\n")
	If InStr(ReConvert,vbCr) Then ReConvert = Replace$(ReConvert,vbCr,"\r")
	If InStr(ReConvert,vbLf) Then ReConvert = Replace$(ReConvert,vbLf,"\n")
	If InStr(ReConvert,vbBack) Then ReConvert = Replace$(ReConvert,vbBack,"\b")
	If InStr(ReConvert,vbFormFeed) Then ReConvert = Replace$(ReConvert,vbFormFeed,"\f")
	If InStr(ReConvert,vbVerticalTab) Then ReConvert = Replace$(ReConvert,vbVerticalTab,"\v")
	If InStr(ReConvert,vbTab) Then ReConvert = Replace$(ReConvert,vbTab,"\t")
	If InStr(ReConvert,vbNullChar) Then ReConvert = Replace$(ReConvert,vbNullChar,"\0")
	ReConvert = ReConvertB(ReConvert)
End Function


'תչַΪʮת
Private Function ReConvertB(ByVal ConverString As String) As String
	Dim i As Long,Length As Long,Dec As Long,Temp As String
	ReConvertB = ConverString
	i = 1
	Do
		Temp = Mid$(ReConvertB,i,1)
		Dec = AscW(Temp)
		If (Dec > 0 And Dec < 32) Or (Dec > 126 And Dec < 256) Then
			ReConvertB = Replace$(ReConvertB,Temp,"\x" & Right$("0" & Hex$(Dec),2))
			i = i + 3
		End If
		Length = Len(ReConvertB)
		i = i + 1
	Loop Until i > Length
End Function


'ִǷַָ(ıͨȽ)
'Mode = 0 ִǷַָҳַָλ
'Mode = 1 ִǷַָֻ
'Mode = 2 ִǷַָ
'Mode = 3 ִǷֻСдַָ
'Mode = 4 ִǷַͬStrNum Ϊַ
'StrRange  ִ鷶Χ ( [Min - Max|Min - Max] ʾΧ)
Public Function CheckStr(ByVal textStr As String,ByVal StrRange As String,Optional ByVal StrNum As Long,Optional ByVal Mode As Long) As Long
	Dim i As Long,Temp As String
	If StrRange = "" Then Exit Function
	If Trim$(textStr) = "" Then Exit Function
	Select Case Mode
	Case 0
		StrRange = Replace$(StrRange,"]|[","")
		If (textStr Like "*" & StrRange & "*") = False Then Exit Function
		For i = 1 To Len(textStr)
			If (Mid$(textStr,i,1) Like StrRange) = True Then
				CheckStr = i
				Exit For
			End If
		Next i
	Case 1
		If (textStr Like "*[!" & Replace$(Replace$(StrRange,"]|[",""),"[","") & "*") = False Then
			CheckStr = True
		End If
	Case 2
		If (textStr Like "*[" & Replace$(Replace$(StrRange,"]|[",""),"[","") & "*") = True Then
			CheckStr = True
		End If
	Case 3
		If InStr(textStr," ") Then Exit Function
		If Len(textStr) < 2 Then Exit Function
		If LCase$(textStr) = textStr Then Exit Function
		If UCase$(textStr) = textStr Then Exit Function
		If (textStr Like "*[!" & Replace$(Replace$(StrRange,"]|[",""),"[","") & "*") = True Then Exit Function
		If (Mid$(textStr,2) Like "*" & Split(StrRange,"|")(0) & "*") = True Then
			If (Mid$(textStr,2) Like "*" & Split(StrRange,"|")(1) & "*") = True Then
				CheckStr = True
			End If
		End If
	Case 4
		If StrNum < 2 Then Exit Function
		If Len(textStr) < StrNum Then Exit Function
		If InStr(textStr," ") Then Exit Function
		If IsNumeric(textStr) = True Then Exit Function
		If IsDate(textStr) = True Then Exit Function
		If (textStr Like "*" & Replace$(StrRange,"]|[","") & "*") = False Then Exit Function
		For i = 1 To Len(textStr)
			Temp = Mid$(textStr,i,1)
			If IsNumeric(Temp) = False Then
				If Temp <> "." Then
					If InStr(textStr,String$(StrNum,Temp)) Then
						CheckStr = True
						Exit For
					End If
				End If
			End If
		Next i
	End Select
End Function


'ַַʮֽڳ
'Mode = 0 ת, ת
Private Function StrHexLength(ByVal textStr As String,ByVal CodePage As Long,ByVal Mode As Long) As Long
	If textStr = "" Then Exit Function
	If Mode = 0 Then textStr = Convert(textStr)
	If CodePage = CP_UNICODELITTLE Or CodePage = CP_UNICODEBIG Then
		'Bin = textStr
		'StrHexLength = UBound(Bin) + 1
		StrHexLength = LenB(textStr)
	Else
		ReDim Bin(0) As Byte
		StrHexLength = WideCharToMultiByte(CodePage, 0&, StrPtr(textStr), Len(textStr), Bin(0), 0, 0, 0)
	End If
End Function


'в
Private Function SplitArgument(ByVal Argument As String,ByVal SplitChar As String,ByVal MaxNum As Long) As String()
	Dim i As Long,j As Long,TempList() As String
	TempList = Split(Argument,SplitChar)
	For i = UBound(TempList) To 0 Step -1
		If IsNumeric(TempList(i)) = False Then Exit For
		j = j + 1
	Next i
	If j = 0 Then
		ReDim TempList(0) As String
		TempList(0) = Argument
	ElseIf UBound(TempList) < MaxNum Then
		For i = 1 To UBound(TempList)
			TempList(i) = Trim$(TempList(i))
		Next i
	Else
		If j > MaxNum Then j = MaxNum - 1
		MaxNum = UBound(TempList)
		For i = 0 To j - 1
			TempList(i + 1) = TempList(MaxNum - j + i + 1)
		Next i
		ReDim Preserve TempList(j) As String
		TempList(0) = ""
		TempList(0) = Split(Argument,Join$(TempList,SplitChar))(0)
		For i = 1 To j
			TempList(i) = Trim$(TempList(i))
		Next i
	End If
	If Trim$(Split(TempList(0),"""")(0) & Split(TempList(0),"""")(UBound(Split(TempList(0),"""")))) = "" Then
		TempList(0) = RemoveBackslash(TempList(0),"""","""",2)
	End If
	SplitArgument = TempList
End Function


'ȥִǰָ PreStr  AppStr
'fType = -1 ȥִǰĿոָ PreStr  AppStrȥִǰո
'fType = 0 ȥִǰĿոָ PreStr  AppStrȥִǰո
'fType = 1 ȥִǰĿոָ PreStr  AppStrȥִǰո
'fType = 2 ȥִǰĿոָ PreStr  AppStr 1 Σȥִǰո
'fType > 2 ȥִǰĿոָ PreStr  AppStr 1 Σȥִǰո
Private Function RemoveBackslash(ByVal Path As String,ByVal PreStr As String,ByVal AppStr As String,ByVal fType As Long) As String
	Dim i As Long,a As Long,p As Long,Stemp As Boolean
	RemoveBackslash = Path
	If Path = "" Then Exit Function
	a = Len(AppStr)
	p = Len(PreStr)
	If fType > -1 Then RemoveBackslash = Trim(RemoveBackslash)
	Do
		Stemp = False
		If p <> 0 Then
			If Left$(RemoveBackslash,p) = PreStr Then
				RemoveBackslash = Mid$(RemoveBackslash,p + 1)
				Stemp = True
			End If
		End If
		If a <> 0 Then
			If Right$(RemoveBackslash,a) = AppStr Then
				RemoveBackslash = Left$(RemoveBackslash,Len(RemoveBackslash) - a)
				Stemp = True
			End If
		End If
		If fType = 1 Or fType > 2 Then RemoveBackslash = Trim$(RemoveBackslash)
		If Stemp = True Then
			If fType < 2 Then i = 0 Else i = 1
		Else
			i = 1
		End If
	Loop Until i = 1
End Function


'תַΪֵ
Private Function StrToLong(ByVal mStr As String,Optional ByVal DefaultValue As Long) As Long
	On Error GoTo errHandle
	StrToLong = CLng(mStr)
	Exit Function
	errHandle:
	StrToLong = DefaultValue
End Function


'ִǷΪգǿշ True
Private Function CheckArray(DataList() As String) As Boolean
	Dim i As Long
	On Error GoTo errHandle
	For i = LBound(DataList) To UBound(DataList)
		If DataList(i) <> "" Then
			CheckArray = True
			Exit For
		End If
	Next i
	errHandle:
End Function


'ʽ HEX ִ
Private Function FormatHexStr(ByVal textStr As String,ByVal Length As Integer) As String
	If textStr = "" Then Exit Function
	If (Len(textStr) Mod Length) = 0 Then
		FormatHexStr = textStr
		Exit Function
	End If
	FormatHexStr = "0" & textStr
End Function


'Ϣ
Private Sub sysErrorMassage(sysError As ErrObject,ByVal fType As Long)
	Dim TempArray() As String,MsgList() As String
	Dim ErrorNumber As Long,ErrorSource As String,ErrorDescription As String
	Dim TitleMsg As String,ContinueMsg As String,Msg As String

	ErrorNumber = sysError.Number
	ErrorSource = sysError.Source
	ErrorDescription = sysError.Description

	TitleMsg = "Error"
	Select Case fType
	Case 0
		ContinueMsg = vbCrLf & vbCrLf & "The program cannot continue and will exit."
	Case 1
		ContinueMsg = vbCrLf & vbCrLf & "Do you want to continue?"
	Case 2
		ContinueMsg = vbCrLf & vbCrLf & "The program will continue to run."
	End Select

	If CheckArray(MsgList) = True Then
		TitleMsg = MsgList(0)
		Select Case fType
		Case 0
			ContinueMsg = MsgList(1)
		Case 1
			ContinueMsg = MsgList(2)
		Case 2
			ContinueMsg = MsgList(3)
		End Select

		Select Case ErrorSource
		Case ""
			If ErrorNumber = 10051 And PSL.Version >= 1500 Then
				Msg = Replace$(MsgList(4),"%s",ErrorSource)
			Else
				Msg = Replace$(Replace$(MsgList(5),"%d",CStr(ErrorNumber)),"%v",ErrorDescription)
			End If
		Case "NotSection"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = Replace$(Replace$(MsgList(6),"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotValue"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = Replace$(Replace$(MsgList(7),"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotReadFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotUnWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotOpenFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotINIFile"
			Msg = Replace$(MsgList(8),"%s",ErrorDescription)
		Case "NotExitFile"
			Msg = Replace$(MsgList(9),"%s",ErrorDescription)
		Case "NotVersion"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = Replace$(MsgList(10),"%s",TempArray(0))
			Msg = Replace$(Replace$(Msg,"%d",TempArray(1)),"%v",TempArray(2))
		Case Else
			Msg = Replace$(MsgList(11),"%s",ErrorSource)
			Msg = Replace$(Replace$(Msg,"%d",CStr(ErrorNumber)),"%v",ErrorDescription)
		End Select
	Else
		Select Case ErrorSource
		Case ""
			If ErrorNumber = 10051 And PSL.Version >= 1500 Then
				Msg = "Unable to open the file. Please verify the file path and file name" & _
						"contains characters in Asian languages. " & vbCrLf & _
						"Note: Passolo 2015 Version of the macro engine does not recognize" & _
						"the file path and file name contains Asian language characters."
			Else
				Msg = "An Error occurred in the program design." & vbCrLf & "Error Code: %d, Content: %v" & _
						vbCrLf & "Please restart the Passolo try and please report to the software developer."
				Msg = Replace$(Replace$(Msg,"%s",CStr(ErrorNumber)),"%v",ErrorDescription)
			End If
		Case "NotSection"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = "The following file is missing [%s] section." & vbCrLf & "%d"
			Msg = Replace$(Replace$(Msg,"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotValue"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = "The following file is missing [%s] Value." & vbCrLf & "%d"
			Msg = Replace$(Replace$(Msg,"%s",TempArray(1)),"%d",TempArray(0))
		Case "NotReadFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotUnWriteFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotOpenFile"
			Msg = Replace$(ErrorDescription,TextJoinStr,vbCrLf)
		Case "NotINIFile"
			Msg = "The following contents of the file is not correct." & vbCrLf & "%s"
			Msg = Replace$(Msg,"%s",ErrorDescription)
		Case "NotExitFile"
			Msg = "The following file does not exist! Please check and try again." & vbCrLf & "%s"
			Msg = Replace$(Msg,"%s",ErrorDescription)
		Case "NotVersion"
			TempArray = Split(ErrorDescription,TextJoinStr,-1)
			Msg = "The following file version is %d, requires version at least %v." & vbCrLf & "%s"
			Msg = Replace$(Msg,"%s",TempArray(0))
			Msg = Replace$(Replace$(Msg,"%d",TempArray(1)),"%v",TempArray(2))
		Case Else
			Msg = "Your system is missing %s server." & vbCrLf & "Error Code: %d, Content: %v"
			Msg = Replace$(Msg,"%s",ErrorSource)
			Msg = Replace$(Replace$(Msg,"%d",CStr(ErrorNumber)),"%v",ErrorDescription)
		End Select
	End If

	If Msg = "" Then Exit Sub
	Select Case fType
	Case 0
		MsgBox(Msg & ContinueMsg,vbOkOnly+vbInformation,TitleMsg)
		Exit All
	Case 1
		If MsgBox(Msg & ContinueMsg,vbYesNo+vbInformation,TitleMsg) = vbNo Then
			Exit All
		End If
	Case Else
		MsgBox(Msg & ContinueMsg,vbOkOnly+vbInformation,TitleMsg)
	End Select
End Sub


'Ϣַ
Private Function GetMsgList(MsgList() As String) As Boolean
	Dim i As Integer
	ReDim MsgList(42) As String
	On Error GoTo errHandle
	MsgList(0) = "Error"
	MsgList(1) = "\r\n\r\nThe program cannot continue and will exit."
	MsgList(2) = "\r\n\r\nDo you want to continue?"
	MsgList(3) = "\r\n\r\nThe program will continue to run."
	MsgList(4) = "Unable to open the file. Please verify the file path and file name" & _
				"contains characters in Asian languages.\r\n" & _
				"Note: Passolo 2015 Version of the macro engine does not recognize" & _
				"the file path and file name contains Asian language characters."
	MsgList(5) = "An Error occurred in the program design.\r\nError Code: %d, Content: %v\r\n" & _
				"Please restart the Passolo try and please report to the software developer."
	MsgList(6) = "The following file is missing [%s] section.\r\n%d"
	MsgList(7) = "The following file is missing [%s] Value.\r\n%d"
	MsgList(8) = "The following contents of the file is not correct.\r\n%s"
	MsgList(9) = "The following file does not exist! Please check and try again.\r\n%s"
	MsgList(10) = "The following file version is %d, requires version at least %v.\r\n%s"
	MsgList(11) = "Your system is missing %s server.\r\nError Code: %d, Content: %v"

	MsgList(12) = "Character Encode Query - Version %v (Build %b)"
	MsgList(13) = "Text:"
	MsgList(14) = "ANSI:"
	MsgList(15) = "Unicode:"
	MsgList(16) = "UniBigEnd:"
	MsgList(17) = "UTF-8:"
	MsgList(18) = "UTF-7:"
	MsgList(19) = "Escape"
	MsgList(20) = "About"
	MsgList(21) = "Copy"
	MsgList(22) = "(%s/%d)"

	MsgList(23) = "OEM"
	MsgList(24) = "MAC"
	MsgList(25) = "THREAD"
	MsgList(26) = "WEST EUROPE"
	MsgList(27) = "EAST EUROPE"
	MsgList(28) = "RUSSIAN"
	MsgList(29) = "GREEK"
	MsgList(30) = "TURKISH"
	MsgList(31) = "HEBREW"
	MsgList(32) = "ARABIC"
	MsgList(33) = "BALTIC"
	MsgList(34) = "VIETNAMESE"
	MsgList(35) = "JAPAN"
	MsgList(36) = "CHINA GB2312"
	MsgList(37) = "CHINA GB18030"
	MsgList(38) = "KOREA"
	MsgList(39) = "TAIWAN"
	MsgList(40) = "THAI"

	MsgList(41) = "Version: %v (Build %b)\r\n" & _
				"OS Version: Windows XP/2000 or higher\r\n" & _
				"Passolo Version: Passolo 5.0 or higher\r\n" & _
				"Copyright: Hanzify\r\nLicense: Freeware\r\n" & _
				"HomePage: http://www.hanzify.org\r\n" & _
				"Author: Hanzify member - wanfu (2014 - 2015)\r\n" & _
				"E-mail: z_shangyi@163.com"
	MsgList(42) = "About Character Encode Query"
	For i = 0 To UBound(MsgList)
		If InStr(MsgList(i),"\\") Then MsgList(i) = Replace$(MsgList(i),"\\","*a!N!d*")
		If InStr(MsgList(i),"\r\n") Then MsgList(i) = Replace$(MsgList(i),"\r\n",vbCrLf)
		If InStr(MsgList(i),"\t") Then MsgList(i) = Replace$(MsgList(i),"\t",vbTab)
		If InStr(MsgList(i),"*a!N!d*") Then MsgList(i) = Replace$(MsgList(i),"*a!N!d*","\")
	Next i
	GetMsgList = True
	Exit Function
	errHandle:
	ReDim MsgList(0) As String
End Function
